(******************************************************************************
 ******************************************************************************
 ***                                                                        ***
 ***  PWGEN2 - a complete rewrite of PWGEN                                  ***
 ***  Version [1.10]                                {Last mod 2012-01-05}   ***
 ***                                                                        ***
 ******************************************************************************
 ******************************************************************************

                                 _\\|//_
                                (` * * ')
 ______________________________ooO_(_)_Ooo_____________________________________
 ******************************************************************************
 ******************************************************************************
 ***                                                                        ***
 ***               Copyright (c) 2003, 2012 by Oliver Schneider             ***
 ***                                                                        ***
 ***   CONTACT TO THE AUTHOR(S):                                            ***
 ***    ____________________________________                                ***
 ***   |                                    |                               ***
 ***   | -=Assarbad=- aka Oliver            |                               ***
 ***   |____________________________________|                               ***
 ***   |                                    |                               ***
 ***   | oliver at assarbad dot net         |                               ***
 ***   | ICQ: 281645                        |                               ***
 ***   |                                    |                               ***
 ***   |____________________________________|                               ***
 ***             ___                                                        ***
 ***            /   |                     ||              ||                ***
 ***           / _  |   ________ ___  ____||__    ___   __||                ***
 ***          / /_\ |  / __/ __//   |/  _/|   \  /   | /   |                ***
 ***         / ___  |__\\__\\  / /\ || |  | /\ \/ /\ |/ /\ | DOT NET        ***
 ***        /_/   \_/___/___/ /_____\|_|  |____/_____\\__/\|                ***
 ***       ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        ***
 ***                        [http://assarbad.net]                           ***
 ***                                                                        ***
 ***   Notes:                                                               ***
 ***   - my first name is Oliver, you may well use this in your e-mails     ***
 ***   - for questions and/or proposals drop me a mail or instant message   ***
 ***                                                                        ***
 ******************************************************************************
 ******************************************************************************

 LEGAL STUFF:
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

 Copyright (c) 2003, 2012 Oliver Schneider ["copyright holder(s)"]
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
 3. The name(s) of the copyright holder(s) may not be used to endorse or
    promote products derived from this software without specific prior written
    permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                             .oooO     Oooo.
 ____________________________(   )_____(   )___________________________________
                              \ (       ) /
                               \_)     (_/

 ******************************************************************************)
program pwgen2;
uses
  Windows,
  CommCtrl,
  Messages;

{$APPTYPE CONSOLE}
{$R main.res}
const
  IDD_DIALOG1 = 101;
  IDD_DIALOG2 = 102;
  IDC_EDIT_PASSWD = 1001;
  IDC_EDIT_CHARS = 1003;
  IDC_RADIO_NUM = 1004;
  IDC_CHECK_AUTOCOPY = 1005;
  IDC_RADIO_ALPHANUM = 1006;
  IDC_RADIO_LETTERS = 1007;
  IDC_EDIT_NUMCHARS = 1008;
  IDC_SPIN_NUMCHARS = 1009;
  IDC_BUTTON_MULTI = 1010;
  IDC_BUTTON_FNAME = 1011;
  IDC_RADIO_SPECIAL = 1011;
  IDC_EDIT_NUMPASSWD = 1012;
  IDC_CHECK_AIGNORECASE = 1012;
  IDC_EDIT_FNAME = 1013;
  IDC_SPIN_NUMPASSWD = 1014;
  IDC_RADIO_SAFESPECIAL = 1015;

  Numeric = '0123456789';
  AlphaOnly = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
  AlphaNumeric = Numeric + AlphaOnly;
  Special = '~!@#$%^&*()-_=+''";:.<>,/?\|{}[]' + AlphaNumeric;
  SaferSpecial = '-_=.,:;!+<>?{}[]/\|' + AlphaNumeric;

  maxlen = sizeof(ShortString);
  cbDefaultLen = 20;

var
  fix8: HFONT;

function GetNumOfUnique(s: string; ignorecase: Boolean): Byte;
var
  a: array[#0..#$FF] of Byte;
  i: Integer;
begin
  ZeroMemory(@a, sizeof(a));
// Default result
  result := 0;
// Check wether string is longer than 0
  if Length(s) > 0 then
  begin
// If ignorecase = True
    if ignorecase then
// ... convert whole string to lower characters
      CharLower(@s[1]);
// Count every character
    for i := 1 to Length(s) do
// ... and increase the respective array member
      inc(a[s[i]]);
// Iterate through the array
    for i := ord(Low(a)) to ord(High(a)) do
// For every entry > 0 ...
      if a[Chr(i)] > 0 then
// ... increase UniqueCounter (i.e. Result)
        inc(Result);
  end;
end;

function _pwgen2(RndSeed: DWORD; NumChars: Byte; PossChars: string; ignorecase: Boolean): string;
var
  c: Char;
const
  maxsleeptime = 3;
begin
  SetLength(PossChars, lstrlen(@PossChars[1]));
  result := '';
// Get number of unique characters ... if not at least two unique -> use default
  if GetNumOfUnique(PossChars, ignorecase) <= 2 then
    PossChars := AlphaNumeric;
// Check for valid RandSeed
  if RndSeed = 0 then
    Randomize
  else
// Set RNG seed value
    Randseed := RndSeed;
// Set minimum NumChars
  if NumChars < 2  then
    NumChars := 2;
// Just wait - done for Random()
  Sleep(Random(maxsleeptime));
// Generate a first character from the given character set
  result := PossChars[Random(Length(PossChars)) + 1];
  repeat
// Sleep - gives better random results
    Sleep(Random(maxsleeptime));
// Take random character from the given character set
    c := PossChars[Random(Length(PossChars)) + 1];
// Check the last character we got
    if ignorecase then
      case CharLower(PChar(Word(c))) = CharLower(PChar(Word(Result[Length(Result)]))) of
// Do not allow same lowercase character twice ... instead continue looping ;)
        True: Continue;
      else
// If different, append it
        result := result + c
      end
    else
      case c = Result[Length(Result)] of
// Do not allow same character twice ... instead continue looping ;)
        True: Continue;
      else
// If different, append it
        result := result + c
      end;
// Try randomizing the bits of RandSeed ;)
    asm
// Save register
      push    EAX
// EAX := RandSeed
      mov     EAX, RandSeed
// EAX := not EAX
      not     EAX
// rotate EAX one bit to the right
      ror     EAX, 1
// RandSeed := EAX
      mov     RandSeed, EAX
// Restore register
      pop     EAX
    end;
  until Length(Result) >= NumChars;
end;

function Multi_DlgFunc(hWnd: HWND; uMsg: UINT; wparam: WPARAM; lparam: LPARAM): BOOL; stdcall;
// DIALOG TO SET EVERYTHING FOR MULTIPLE PASSWORDS
begin
  Result := True;
  case uMsg of
    WM_INITDIALOG:
      begin
      end;
    WM_CLOSE:
// Close the dialog
      EndDialog(hWnd, 0);
    WM_COMMAND:
      case HiWord(WParam) of
// A button was clicked
        BN_CLICKED:
          case LOWORD(wParam) of
// OK Buttin is there to generate passwords
            IDOK:
              begin
              end;
            IDCANCEL:
              SendMessage(hwnd, WM_CLOSE, 0, 0);
          end;
      end;
  else
    Result := False;
  end;
end;

function Main_EnumChildSetFonts(hwnd: HWND; lParam: LPARAM): BOOL; stdcall;
begin
  result := True;
// According to the current control ...
  case GetDlgCtrlID(hwnd) of
//  IDC_EDIT_CHARS,
    IDC_EDIT_NUMCHARS,
      IDC_EDIT_PASSWD:
// ... set the font given by lParam
      SendMessage(hwnd, WM_SETFONT, lParam, ord(TRUE));
  end;
// According to the current control ...
  case GetDlgCtrlID(hwnd) of
    IDC_EDIT_CHARS,
      IDC_EDIT_PASSWD:
// ... limit its text length!
      SendMessage(hwnd, EM_SETLIMITTEXT, $FF, 0);
  end;
end;

procedure Main_ControlsDisabled(hwnd: HWND; Disabled: BOOL);
begin
  EnableWindow(GetDlgItem(hwnd, IDC_RADIO_ALPHANUM), not Disabled);
  EnableWindow(GetDlgItem(hwnd, IDC_RADIO_NUM), not Disabled);
  EnableWindow(GetDlgItem(hwnd, IDC_RADIO_LETTERS), not Disabled);
  EnableWindow(GetDlgItem(hwnd, IDC_RADIO_SPECIAL), not Disabled);
  EnableWindow(GetDlgItem(hwnd, IDC_CHECK_AUTOCOPY), not Disabled);
  EnableWindow(GetDlgItem(hwnd, IDC_CHECK_AIGNORECASE), not Disabled);
  EnableWindow(GetDlgItem(hwnd, IDC_EDIT_CHARS), not Disabled);
  EnableWindow(GetDlgItem(hwnd, IDOK), not Disabled);
  EnableWindow(GetDlgItem(hwnd, IDC_BUTTON_MULTI), not Disabled);
end;

function Main_DlgFunc(hWnd: HWND; uMsg: UINT; wparam: WPARAM; lparam: LPARAM): BOOL; stdcall;
// MAIN DIALOG
var
  temps,
    characters,
    passwd: string;
  len: Integer;
begin
  Result := True;
  case uMsg of
    WM_INITDIALOG:
      begin
// Get a 8 Point fixed font
        fix8 := CreateFont(-MulDiv(8, GetDeviceCaps(GetWindowDC(hwnd), LOGPIXELSY), 72), 0, 0, 0, FW_NORMAL, 0, 0, 0, ANSI_CHARSET, OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS, PROOF_QUALITY, FIXED_PITCH or FF_MODERN, 'Courier New');
// And set it
        EnumChildWindows(hwnd, @Main_EnumChildSetFonts, fix8);
// Set default character set
        SetDlgItemText(hwnd, IDC_EDIT_CHARS, SaferSpecial);
// Default for radio buttons
        CheckDlgButton(hwnd, IDC_RADIO_SAFESPECIAL, BST_CHECKED);
// Default for checkboxes
        CheckDlgButton(hwnd, IDC_CHECK_AUTOCOPY, BST_CHECKED);
        CheckDlgButton(hwnd, IDC_CHECK_AIGNORECASE, BST_CHECKED);
// Set default and range for the up-down control (spin control)
        SendDlgItemMessage(hwnd, IDC_SPIN_NUMCHARS, UDM_SETRANGE, 0, MAKELONG($FF, 1));
        SendDlgItemMessage(hwnd, IDC_SPIN_NUMCHARS, UDM_SETPOS, 0, MAKELONG(cbDefaultLen, 0));
// MAKE MULTI-BUTTON INVISIBLE
        SetWindowLong(
        GetDlgItem(hwnd, IDC_BUTTON_MULTI),
         GWL_STYLE,
         GetWindowLong(GetDlgItem(hwnd, IDC_BUTTON_MULTI), GWL_STYLE) and (not WS_VISIBLE)
         );
      end;
    WM_CLOSE:
// Close the dialog
      EndDialog(hWnd, 0);
    WM_COMMAND:
      case HiWord(WParam) of
// A button was clicked
        BN_CLICKED:
          case LOWORD(wParam) of
// OK Buttin is there to generate one password
            IDOK:
              begin
                Main_ControlsDisabled(hwnd, True);
                len := GetWindowTextLength(GetDlgItem(hwnd, IDC_EDIT_CHARS));
                if len > maxlen + 2 then
                  len := maxlen + 2;
                SetLength(characters, len);
                ZeroMemory(@characters[1], len);
// Maximum length is $FF-1 !!!
                GetDlgItemText(hwnd, IDC_EDIT_CHARS, @characters[1], len);
// Get a password
                passwd := _pwgen2(
                  0,
                  SendDlgItemMessage(hwnd, IDC_SPIN_NUMCHARS, UDM_GETPOS, 0, 0),
                  characters,
                  IsDlgButtonChecked(hwnd, IDC_CHECK_AIGNORECASE) = BST_CHECKED
                  );
// Set the password into the EDIT
                SetDlgItemText(hwnd, IDC_EDIT_PASSWD, @passwd[1]);
// Check wether the user wishes to have the password copied into the clipboard
                if IsDlgButtonChecked(hwnd, IDC_CHECK_AUTOCOPY) = BST_CHECKED then
                begin
// Mark whole text
                  SendDlgItemMessage(hwnd, IDC_EDIT_PASSWD, EM_SETSEL, 0, -1);
// Tell the EDIT to copy selected contents to clipboard
                  SendDlgItemMessage(hwnd, IDC_EDIT_PASSWD, WM_COPY, 0, 0);
// Mark no text
                  SendDlgItemMessage(hwnd, IDC_EDIT_PASSWD, EM_SETSEL, -1, 0);
                end;
                Main_ControlsDisabled(hwnd, False);
              end;
// Set the text upon click onto one of the radio buttons
            IDC_RADIO_NUM,
              IDC_RADIO_ALPHANUM,
              IDC_RADIO_LETTERS,
              IDC_RADIO_SAFESPECIAL,
              IDC_RADIO_SPECIAL:
              begin
                case LOWORD(wParam) of
                  IDC_RADIO_NUM:
                    temps := Numeric;
                  IDC_RADIO_LETTERS:
                    temps := AlphaOnly;
                  IDC_RADIO_ALPHANUM:
                    temps := AlphaNumeric;
                  IDC_RADIO_SPECIAL:
                    temps := Special;
                else // IDC_RADIO_SAFESPECIAL:
                  temps := SaferSpecial;
                end;
                SetDlgItemText(hwnd, IDC_EDIT_CHARS, @temps[1]);
              end;
            IDC_BUTTON_MULTI:
              DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_DIALOG2), hwnd, @Multi_DlgFunc, 0);
          end;
      end;
  else
// Handle all other window messages
    Result := False;
  end;
end;

procedure ConHelp;
begin
  Writeln(#13#10);
  Writeln('PWGEN2 - Copyright (c) 2003, 2012 by Oliver Schneider'#13#10);
  Writeln('Syntax:');
  Writeln('    <No parameters>  Starts windowed mode (GUI)');
  Writeln('    /?               Shows this help screen');
  Writeln('    CON              Writes output password to console'#13#10);
  Writeln('Please note that the password in console mode is always alpha-numerical and');
  Writeln('has exactly 20 characters length. This is by intention. Source code for this');
  Writeln('program is available, so in case you need -> modify the source!'#13#10);
  Writeln('A sample script for console mode (uses CMD.EXE!) should be in the archive.');
  Writeln('If not, just get a new copy from:');
  Writeln('------------------------------------------------------------------------------');
  Writeln('                          http://assarbad.net/stuff');
  Writeln('------------------------------------------------------------------------------');
  Writeln;

end;

begin
// Compare first parameter of module case-insensitive with 'CON' - for console mode
  if lstrcmpi(@paramstr(1)[1], 'CON') = 0 then
// Default for console password is ALWAYS 20 and default character set is alpha-numeric
    Write(_pwgen2(GetTickCount, cbDefaultLen, AlphaNumeric, True))
  else
    if lstrcmpi(@paramstr(1)[1], '/?') = 0 then
// Default for console password is ALWAYS 20 and default character set is alpha-numeric
      ConHelp()
    else
    begin
// This is not console mode ...
      FreeConsole();
// Init common controls DLL (we use up-down controls)
      InitCommonControls();
// Create and show main dialog
      DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_DIALOG1), 0, @Main_DlgFunc, 0);
    end;
end.

